<?php

namespace Bydedi;

use DomDocument;
use DomXPath;

/**
 * Class Vidal
 * @package Bydedi
 */
class Vidal {

    public $xmlSearch = '';
    public $xmlInteraction = '';
    public $xmlPackage = array();
    public $xml = '';
    public $idPackageVidal = null;
    public $id = null;
    public $linkFicheVidalFamille = '';
    public $linkFicheMono = '';
    public $linkProduit = '';
    public $urlWebService = '';

    public $properties = '';

    /**
     * @var bool
     */
    protected static $init = false;
    /**
     * @var
     */
    protected static $config;

    /*
     * Use ton configure your API
     */
    public static function config($config = array())
    {
        if (self::$init == false) {
            $default = [
                'credentials' => [
                    'app_id' => 'xxx',
                    'app_key' => 'xxx',
                ],
                'urlWebService' => 'http://api.vidal.fr',
                'xmlSearch' => 'http://api.vidal.fr/rest/api/search?code=%s',
                'xmlInteraction' => 'http://api.vidal.fr/rest/api/alerts/interactions?productId=%s&productId=%s',
                'xmlPackage' => 'http://api.vidal.fr/rest/api/package/%s/',
                'xmlDocument' => 'http://api.vidal.fr/rest/api/product/%s/documents',
                'xmlProduct' => 'http://api.vidal.fr/rest/api/products?page_size=%i&q=%s',
                'listProperties' => [
                    'composition' => 'produit_nom_desc_composition',
                    'resultats' => 'produit_nom_desc_resultats',
                    'posologie' => 'produit_nom_desc_posologie',
                    'contre_indication' => 'produit_nom_desc_contre_indication',
                    'precautions' => 'produit_nom_desc_precautions',
                    'precautionsbis' => 'produit_nom_desc_precautions',
                    'enceinte' => 'produit_nom_desc_enceinte',
                    'utilisation' => 'produit_nom_desc_utilisation',
                    'conseil' => 'produit_nom_desc_conseil',
                    'effet_indesirable' => 'produit_nom_desc_effet_indesirable'
                ],
                'regexUse' => false,
                'regex' => [
                    'composition' => [
                        'key' => '|<td align="[a-z]+">(.*)<\/[^>]+>|U',
                        'value' => '|<td width="[0-9]+%" align="[a-z]+">(.*)<\/[^>]+>|U'
                    ],
                    'contre_indication' => [
                        'title' => '|(.*)<ul>|',
                        'value' => '|<li type="[a-z]+">(.*)<\/[^>]+>|U',
                    ],
                    'enceinte' => [
                        'key' => '|<b>(.*)<\/b>|U',
                        'value' => '|<br>([^\s]+.*)<br>|U'
                    ],
                    'posologie' => [
                        'title' => '|(.*)[\s]*<br>[\s]|U',
                        'value' => '|<p[\s><]+br>[\s]*(.*)[\s]*<br[\s><\/]+p>|',
                        'precision' => '|<\/p>(.*)|'
                    ],
                    'precautions' => [
                        'title' => '|(.*)<ul>|',
                        'value' => '|<li type="[a-z]+">(.*)<\/[li]+>|U',
                        'precision' => '|<\/ul>(.*)|'
                    ]
                ],

            ];
            self::$config = array_replace_recursive($default, $config);
            self::$init = true;
        }
    }

    public function __construct($cip13 = null)
    {
        if (!is_null($cip13)) {
            $this->load($cip13);
        }

    }

    /**
     * Make call xith Vidal's API
     * Add "app_id" & "app_key" to your url request
     *
     * @param $url
     *
     * @return SimpleXMLElement
     */
    protected function makeCall($url)
    {
        $charUrl = (strpos($url, '?') === false) ? '?' : '&';
        $url.= $charUrl . http_build_query(self::$config['credentials']);
        $result = simplexml_load_file($url);

        return $result;
    }

    /**
     * Load product informations
     * @param null $cip13 - cip13 code of the vidal's product
     * @return bool
     */
    public function load($cip13 = null)
    {
        $this->produit_cip13 = $cip13;
        $this->getLoadFile();

        if (empty($this->xml)) {
            return false;
        }

        foreach ($this->xml->entry as $entry) {
            $namespaces = $entry->getNameSpaces(true);
            if (!empty($namespaces['vidal'])) {
                $arrDataVidal = $entry->children($namespaces['vidal']);
                if (!empty($arrDataVidal->id)) {
                    $this->idPackageVidal = (String) $arrDataVidal->id;
                }
                $xmlPackage = $this->makeCall(sprintf(self::$config['xmlPackage'], $this->idPackageVidal));

                foreach ($xmlPackage->entry as $entry2) {
                    $arrLink = array();
                    foreach ($entry2->link as $links) {
                        $arrTemp = array();
                        foreach ($links->attributes() as $k => $link) {

                            $arrTemp[$k] = (String) $link;
                        }
                        $arrLink[] = $arrTemp;
                    }
                }

                foreach ($arrLink as $link) {
                    if (!empty($link['title']) && $link['title'] == 'PRODUCT') {
                        $this->linkProduit = $this->makeCall(self::$config['urlWebService'] . $link['href']);

                    }
                }
                foreach ($this->linkProduit->entry as $detailProd) {
                    $namespaces = $detailProd->getNameSpaces(true);
                    if (!empty($namespaces['vidal'])) {
                        $arrDataVidal = $detailProd->children($namespaces['vidal']);
                        if (!empty($arrDataVidal)) {
                            $this->id = (String) $arrDataVidal->id;
                            foreach($arrDataVidal as $kV => $vV) {
                                $this->properties[$kV] = (String)$vV;
                            }
                        }
                    }
                }


                if (!empty($this->id)) {
                    $xmlDocProduit = $this->makeCall(sprintf(self::$config['xmlDocument'], $this->id));
                    $arrDoc = array();
                    foreach ($xmlDocProduit->entry as $detailProd) {
                        foreach ($detailProd as $det) {
                            foreach ($det->attributes() as $k => $link) {
                                $arrTemp[$k] = (String)$link;
                            }
                            $arrDoc[] = $arrTemp;
                        }
                    }
                }

                foreach ($arrDoc as $link) {
                    if ($link['title'] == 'VDF') {
                        $this->linkFicheVidalFamille = $link['href'];
                        $this->properties['notice_famille'] = $link['href'];

                    }
                    if ($link['title'] == 'MONO') {
                        $this->linkFicheMono = $link['href'];
                        $this->properties['notice'] = $link['href'];
                    }
                }
            }
        }

        $this->getBase();
        $this->getFeatures();
        $this->properties = array_merge($this->properties, $this->getInfoNom());
        $this->hydrate();

        return true;
    }

    public function getLoadFile()
    {
        if (!empty($this->produit_cip13)) {
            $this->xml = $this->makeCall(sprintf(self::$config['xmlSearch'], $this->produit_cip13));
        }
        return true;
    }

    /*
     * Get Base informations about vidal product
     */
    protected function getBase()
    {
        foreach ($this->xml->entry as $entry) {
            $this->properties['produit_nom_nom'] = (String) $entry->title;
            $arrdate = explode('T', (String) $entry->updated);
            $arrDonnees['produit_date_maj_vidal'] = $arrdate[0];
            $namespaces = $entry->getNameSpaces(true);
            if (!empty($namespaces['vidal'])) {
                $arrDataVidal = $entry->children($namespaces['vidal']);
                if (!empty($arrDataVidal)) {
                    $this->idPackageVidal = $arrDataVidal->id;
                    $this->properties['produit_cip7'] = (String) $arrDataVidal->cip;
                    $this->properties['produit_cip13'] = (String) $arrDataVidal->cip13;
                    $this->properties['produit_cis'] = $arrDataVidal->cis;
                    $this->properties['produit_otc'] = (boolean) $arrDataVidal->otc;
                }
            }
        }
    }

    public function getInfoNom()
    {

        if(!empty($this->linkFicheVidalFamille)){
            return $this->getInfoNomVidal();
        }elseif(!empty($this->linkFicheMono)){
            return $this->getInfosNomMono();
        }else{
            return false;
        }
    }

    public function getInfoNomVidal()
    {
        $arrParamProd = array();
        $file = file_get_contents($this->linkFicheVidalFamille);
        $file = preg_replace("#<style(.*?)>(.*?)</style>#si", "", $file);
        $arrDonnees = explode('<font face="Arial, Helvetica, sans-serif" color="#bb1100">', $file);
        $arrData = array();
        foreach ($arrDonnees as $d) {
            preg_match("#<a name=\"(.*?)\">#i", $d, $match);
            preg_match("#</h5>(.*?)<h5>#si", $d, $desc);
            if (empty($desc[1])) {
                preg_match("#</h5>(.*?)<hr>#si", $d, $desc);
            }
            if (!empty($desc[1]) && !empty($match[1])) {
                $desc[1] = preg_replace("#<\/?a(\s+.*?>|>)#i", "", $desc[1]);
                $desc[1] = preg_replace("#<\/?span(\s+.*?>|>)#i", "", $desc[1]);
                $desc[1] = preg_replace("#<\/?font(\s+.*?>|>)#i", "", $desc[1]);
                $arrData[$match[1]] = $desc[1];
            }
        }

        // init
        $arrParamProd = [];


        //fin init
        foreach ($arrData as $k => $data) {
            switch ($k) {
                // PRÉSENTATIONS
                case 'forme':
                    continue;
                    break;
                // COMPOSITION
                case 'compo':
                    $arrParamProd[self::$config['listProperties']['composition']] = $this->getValues($data, 'composition');
                    break;
                // INDICATIONS
                case 'indic':
                    $arrParamProd[self::$config['listProperties']['resultats']] = $this->getValues($data, 'resultats');
                    break;
                // CONTRE-INDICATIONS
                case 'contrind':
//                    preg_match_all(self::$config['regex']['contre_indication']['value'], preg_replace("/(\r\n|\n|\r|<br>)/", "", $data), $values, PREG_PATTERN_ORDER);
                    $arrParamProd[self::$config['listProperties']['contre_indication']] = $this->getValues($data, 'contre_indication');
                    break;
                // ATTENTION
                case 'megpe':
                    $arrParamProd[self::$config['listProperties']['utilisation']] = $this->getValues($data, 'utilisation');
                    break;
                // INTERACTIONS MEDICAMENTEUSES
                case 'inter':
                    $arrParamProd[self::$config['listProperties']['precautions']] = $this->getValues($data, 'precautions');
                    break;
                // GROSSESSE et ALLAITEMENT
                case 'gross':
                    $arrParamProd[self::$config['listProperties']['enceinte']] = $this->getValues($data, 'enceinte');
                    break;
                // MODE D'EMPLOI ET POSOLOGIE
                case 'posol':
                    $arrParamProd[self::$config['listProperties']['posologie']] = $this->getValues($data, 'posologie');
                    break;
                // CONSEILS
                case 'consei':
                    $arrParamProd[self::$config['listProperties']['conseil']] = $this->getValues($data, 'conseil');
                    break;
                // EFFETS INDÉSIRABLES POSSIBLES
                case 'effet':
                    $arrParamProd[self::$config['listProperties']['effet_indesirable']] = $this->getValues($data, 'effet_indesirable');
                    break;
                default :
                    continue;
                    break;
            }
        }
        return $arrParamProd;
    }

    public function getInfosNomMono()
    {
        $arrParamProd = array();

        $dom = new DomDocument();
        $dom->loadHTMLFile($this->linkFicheMono);
        $finder = new DomXPath($dom);
        $classname = "AmmComposition";
        $nodesCompo = $finder->query("//*[contains(@class, '$classname')]");

        foreach ($nodesCompo as $n) {
            $arrParamProd[self::$config['listProperties']['composition']][] = utf8_decode(trim($n->nodeValue));
        }

        $classname = 'AmmCorpsTexte';
        $nodesDesc = $finder->query("//*[contains(@class, '$classname')]");

        $arrIndex = array_values(self::$config['listProperties']);

        foreach ($nodesDesc as $k => $n) {
            if(!empty($n->nodeValue)) {
                if (array_key_exists(($k + 1), $arrIndex)) {
                    $arrParamProd[$arrIndex[$k + 1]][] = utf8_decode(trim($n->nodeValue));
                }
            }
        }
        return $arrParamProd;
    }

    /**
     * GEt product features
     * @param array $formvars
     * @param string $champs
     * @return bool
     */
    public function getFeatures()
    {
        if(empty($this->linkProduit)){
            return false;
        }
        foreach ($this->linkProduit->entry as $detailProd) {
            $namespaces = $detailProd->getNameSpaces(true);
            if (!empty($namespaces['vidal'])) {
                $arrDataVidal = $detailProd->children($namespaces['vidal']);
                if (!empty($arrDataVidal)) {
                    $this->properties['caracteristique']['company'] = (String) $arrDataVidal->company;
                    if ((String) $arrDataVidal->beCareful == 'true') {
                        $this->properties['caracteristique']['beCareful'] = 'OUI';
                    } elseif ((String) $arrDataVidal->beCareful == 'false') {
                        $this->properties['caracteristique']['beCareful'] = 'NON';
                    }
                    if ((String) $arrDataVidal->drugInSport == 'true') {
                        $this->properties['caracteristique']['drugInSport'] = 'OUI';
                    } elseif ((String) $arrDataVidal->drugInSport == 'false') {
                        $this->properties['caracteristique']['drugInSport'] = 'NON';
                    }
                    $this->properties['caracteristique']['galenicForm'] = (String) $arrDataVidal->galenicForm;

                    if ((String) $arrDataVidal->withoutPrescription == 'true') {
                        $this->properties['caracteristique']['withoutPrescription'] = 'OUI';
                    } elseif ((String) $arrDataVidal->withoutPrescription == 'false') {
                        $this->properties['caracteristique']['withoutPrescription'] = 'NON';
                    }

                    $this->properties['caracteristique']['perVolume'] = (String) $arrDataVidal->perVolume;
                    $this->properties['caracteristique']['vigilance'] = (String) $arrDataVidal->vigilance;
                }
            }
        }
    }

    public function importCaracVidal($formvars = array())
    {
        $q = new Query();
        $cara = array();
        if (!empty($formvars['caracteristique_nom'])) {
            $cara['caracteristique_nom'] = $formvars['caracteristique_nom'];
            if ($cara['caracteristique_nom'] == 'company') {
                $cara['caracteristique_nom'] = 'Marques';
            }
        } elseif (!empty($formvars['caracteristique_id'])) {
            $cara['caracteristique_id'] = $formvars['caracteristique_id'];
        } else {
            return false;
        }
        if (empty($formvars['caracteristique_valeur_nom'])) {
            return false;
        }
        $cara['caracteristique_valeur_nom'] = $formvars['caracteristique_valeur_nom'];
        if (is_bool($cara['caracteristique_valeur_nom']) && $cara['caracteristique_valeur_nom'] == true) {
            $cara['caracteristique_valeur_nom'] = 'OUI';
        } elseif (is_bool($cara['caracteristique_valeur_nom']) && $cara['caracteristique_valeur_nom'] == false) {
            $cara['caracteristique_valeur_nom'] = 'NON';
        }

        return $cara;
    }

    public function getUrlFicheFamille($formvars = array(), $champs = "")
    {
        return $this->linkFicheVidalFamille;
    }

    public function searchProduitVidal($formvars = array(), $champs = "")
    {
        $xml = $this->makeCall(sprintf(self::$config['xmlProduct'], 100, $formvars['mot_cle']));
        return $xml;
    }

    public function getInteraction($id1 = null, $id2 = null)
    {
        $arrAlert = array();
        if (!empty($id1) && !empty($id2)) {
            $urlInteraction = sprintf(self::$config['xmlInteraction'], $id1, $id2);
            $xmlInteraction = $this->makeCall($urlInteraction);
            if (count($xmlInteraction->entry) > 2) {
                foreach ($xmlInteraction->entry as $entry) {
                    foreach ($entry->category->attributes() as $categorie) {
                        $cat = (String) $categorie;
                        if ($cat == 'ALERT' && (empty($arrAlert) || strlen((String) $entry->content) > strlen($arrAlert['description']))) {
                            $arrAlert['title'] = str_replace('Interaction médicamenteuse : ', '', (String) $entry->title);
                            $arrAlert['source'] = (String) $entry->contributor->name;
                            $arrAlert['description'] = str_replace(array('<span>', '</span>'), array('<br/><span style="font-weight:bold;text-decoration:underline;">', '</span><br/>'), (String) $entry->content);
                            $arrAlert['idInteractionVidal'] = (String) $entry->id;
                        }
                    }
                    $arrDonnees['produit_nom_nom'] = (String) $entry->title;
                }
            } else {
                $arrAlert = array();
            }
        }

        return $arrAlert;
    }

    private function getValues($datas, $regexName)
    {
        if (self::$config['regexUse'] == false) {
            return $datas;
        }

        if(!array_key_exists($regexName, self::$config['regex'])) {
            return [
                $datas
            ];
        }

//        $datas = preg_replace("/(\r\n|\n|\r)/", "", $datas);

        $array = [];
        foreach (self::$config['regex'][$regexName] as $key => $regex) {
            preg_match_all($regex, $datas, $array[$key], PREG_PATTERN_ORDER);
        }

        switch(true) {
            case array_key_exists('key', $array):
                $keys = $array['key'][1];
                $values = $array['value'][1];
                $c = 0;
                $array = [];
                foreach($keys as $k => $key) {
                    if (isset($values[$k])) {
                        $keys[$k] = str_replace("&nbsp;",'',$keys[$k]);
                        $values[$k] = str_replace("&nbsp;",'',$values[$k]);
                        $array[(empty($keys[$k])) ? $c : $keys[$k]] = $values[$k];
                    }
                    $c++;
                }
                break;
            case array_key_exists('title', $array):
            case array_key_exists('precision', $array):
                $subArray = [];
                foreach($array as $keyA => $value) {
                    foreach($value[1] as $k => $v) {
                        if($keyA == 'title' && $k > 0) {
                            unset($value[1][$k]);
                        } else {
                            $value[1][$k] = strip_tags($v);
                        }
                    }
                    $subArray = array_merge($subArray, $value[1]);
                }
                $array = $subArray;
                break;
            default:
                $array = $array['value'][1];
        }

        return $array;

    }

    /**
     * Transform properties to object vars
     */
    public function hydrate()
    {
        if (!empty($this->properties)) {
            foreach ($this->properties as $key => $property) {
                $this->$key = $property;
            }
        }
    }

    public function toArray()
    {
        return $this->properties;
    }

}